@@ -369,8 +369,8 @@ impl<T, const N: usize> StaticVec<T, N> {
     unsafe { self.mut_ptr_at_unchecked(index) }
   }
 
-  /// Returns a constant reference to the element of the StaticVec at `index` if `index` is within
-  /// the range `0..self.length`, without performing any checks to ensure that is actually the case.
+  /// Returns a constant reference to the element of the StaticVec at `index` without doing any
+  /// checking to ensure that `index` is actually within any particular bounds.
   ///
   /// Note that unlike [`slice::get_unchecked`](https://doc.rust-lang.org/nightly/std/primitive.slice.html#method.get_unchecked),
   /// this method only supports accessing individual elements via `usize`; it cannot also produce
@@ -379,34 +379,38 @@ impl<T, const N: usize> StaticVec<T, N> {
   ///
   /// # Safety
   ///
-  /// It is up to the caller to ensure that `index` is within the appropriate bounds.
+  /// It is up to the caller to ensure that `index` is within the range `0..self.length`.
   #[inline(always)]
   pub unsafe fn get_unchecked(&self, index: usize) -> &T {
+    // This function is used internally in places where `length` has been intentionally
+    // temporarily set to zero, so we do our debug assertion against `N`.
     debug_assert!(
-      index < self.length,
+      index < N,
       "In `StaticVec::get_unchecked`, provided index {} must be within `0..{}`!",
       index,
-      self.length
+      N
     );
     &*self.ptr_at_unchecked(index)
   }
 
-  /// Returns a mutable reference to the element of the StaticVec at `index` if `index` is within
-  /// the range `0..self.length`, without performing any checks to ensure that is actually the case.
+  /// Returns a mutable reference to the element of the StaticVec at `index` without doing any
+  /// checking to ensure that `index` is actually within any particular bounds.
   ///
   /// The same differences between this method and the slice method of the same name
   /// apply as do for [`get_unchecked`](crate::StaticVec::get_unchecked).
   ///
   /// # Safety
   ///
-  /// It is up to the caller to ensure that `index` is within the appropriate bounds.
+  /// It is up to the caller to ensure that `index` is within the range `0..self.length`.
   #[inline(always)]
   pub unsafe fn get_unchecked_mut(&mut self, index: usize) -> &mut T {
+    // This function is used internally in places where `length` has been intentionally
+    // temporarily set to zero, so we do our debug assertion against `N`.
     debug_assert!(
-      index < self.length,
+      index < N,
       "In `StaticVec::get_unchecked_mut`, provided index {} must be within `0..{}`!",
       index,
-      self.length
+      N
     );
     &mut *self.mut_ptr_at_unchecked(index)
   }
@@ -467,7 +471,10 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// full; that is, if `self.len() == self.capacity()`.
   #[inline(always)]
   pub fn push(&mut self, value: T) {
-    assert!(self.is_not_full());
+    assert!(
+      self.is_not_full(),
+      "`StaticVec::push` was called through a StaticVec already at maximum capacity!"
+    );
     unsafe { self.push_unchecked(value) };
   }
 
@@ -558,7 +565,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   }
 
   /// Returns `None` if `index` is greater than or equal to the current length of the StaticVec.
-  /// otherwise, removes the value at that position and returns it in `Some`, and then
+  /// Otherwise, removes the value at that position and returns it in `Some`, and then
   /// moves the last value in the StaticVec into the empty slot.
   #[inline(always)]
   pub fn swap_pop(&mut self, index: usize) -> Option<T> {
@@ -1121,10 +1128,10 @@ impl<T, const N: usize> StaticVec<T, N> {
         start: end,
         length: length - end,
         iter: StaticVecIterConst {
-          start: self.mut_ptr_at_unchecked(start),
+          start: self.ptr_at_unchecked(start),
           end: match intrinsics::size_of::<T>() {
-            0 => (self.as_mut_ptr() as *mut u8).wrapping_add(end) as *mut T,
-            _ => self.mut_ptr_at_unchecked(end),
+            0 => (self.as_ptr() as *const u8).wrapping_add(end) as *const T,
+            _ => self.ptr_at_unchecked(end),
           },
           marker: PhantomData,
         },
@@ -1138,27 +1145,30 @@ impl<T, const N: usize> StaticVec<T, N> {
   #[inline]
   pub fn drain_filter<F>(&mut self, mut filter: F) -> Self
   where F: FnMut(&mut T) -> bool {
-    let mut res = Self::new();
     let old_length = self.length;
     // Temporarily set our length to 0 to avoid double drops and such if anything
     // goes wrong in the filter loop.
     self.length = 0;
+    let mut res = Self::new();
+    let mut res_length = 0;
     unsafe {
       // If `self.length` was already 0, this loop is skipped completely.
       for i in 0..old_length {
         // This is fine because we intentionally set `self.length` to `0` ourselves just now.
-        let val = self.mut_ptr_at_unchecked(i);
-        if filter(&mut *val) {
-          res.mut_ptr_at_unchecked(res.length).write(val.read());
-          res.length += 1;
-        } else if res.length > 0 {
+        if filter(self.get_unchecked_mut(i)) {
+          res
+            .mut_ptr_at_unchecked(res_length)
+            .write(self.ptr_at_unchecked(i).read());
+          res_length += 1;
+        } else if res_length > 0 {
           self
             .ptr_at_unchecked(i)
-            .copy_to_nonoverlapping(self.mut_ptr_at_unchecked(i - res.length), 1);
+            .copy_to_nonoverlapping(self.mut_ptr_at_unchecked(i - res_length), 1);
         }
       }
     }
-    self.length = old_length - res.length;
+    self.length = old_length - res_length;
+    res.length = res_length;
     res
   }
 
