@@ -1,6 +1,7 @@
 use crate::iterators::*;
 use crate::utils::{make_const_slice, make_mut_slice, partial_compare};
 use crate::StaticVec;
+use core::borrow::{Borrow, BorrowMut};
 use core::cmp::{Eq, Ord, Ordering, PartialEq};
 use core::fmt::{self, Debug, Formatter};
 use core::hash::{Hash, Hasher};
@@ -49,6 +50,20 @@ impl<T, const N: usize> AsRef<[T]> for StaticVec<T, N> {
   }
 }
 
+impl<T, const N: usize> Borrow<[T]> for StaticVec<T, N> {
+  #[inline(always)]
+  fn borrow(&self) -> &[T] {
+    &self[..]
+  }
+}
+
+impl<T, const N: usize> BorrowMut<[T]> for StaticVec<T, N> {
+  #[inline(always)]
+  fn borrow_mut(&mut self) -> &mut [T] {
+    &mut self[..]
+  }
+}
+
 impl<T: Clone, const N: usize> Clone for StaticVec<T, N> {
   #[inline]
   default fn clone(&self) -> Self {
@@ -381,7 +396,12 @@ impl<T, const N: usize> Index<usize> for StaticVec<T, N> {
   /// and if so returns the value at that position as a constant reference.
   #[inline(always)]
   fn index(&self, index: usize) -> &Self::Output {
-    assert!(index < self.length);
+    assert!(
+      index < self.length,
+      "In StaticVec::index, provided index {} must be less than the current length of {}!",
+      index,
+      self.length
+    );
     unsafe { self.get_unchecked(index) }
   }
 }
@@ -391,7 +411,12 @@ impl<T, const N: usize> IndexMut<usize> for StaticVec<T, N> {
   /// and if so returns the value at that position as a mutable reference.
   #[inline(always)]
   fn index_mut(&mut self, index: usize) -> &mut Self::Output {
-    assert!(index < self.length);
+    assert!(
+      index < self.length,
+      "In StaticVec::index_mut, provided index {} must be less than the current length of {}!",
+      index,
+      self.length
+    );
     unsafe { self.get_unchecked_mut(index) }
   }
 }
@@ -667,12 +692,14 @@ impl<const N: usize> Read for StaticVec<u8, N> {
         .copy_from_nonoverlapping(self.as_ptr(), read_length);
     }
     if read_length < current_length {
-      // TODO: find out if the optimizer elides the bounds check here. It
-      // should be able to, since the only non-const value is read_length,
-      // which is known to be <= self.length
-      self.as_mut_slice().copy_within(read_length.., 0);
+      // Safety: we just confirmed that read_length is less than our current length.
+      unsafe {
+        self
+          .ptr_at_unchecked(read_length)
+          .copy_to(self.as_mut_ptr(), current_length - read_length)
+      };
     }
-    // Safety: 0 <= read_length <= self.length
+    // Safety: 0 <= read_length <= current_length
     unsafe { self.set_len(current_length - read_length) };
     Ok(read_length)
   }
@@ -715,7 +742,7 @@ impl<const N: usize> Read for StaticVec<u8, N> {
     // internal data only once. This as opposed to calling `read` in a loop,
     // which shifts the inner data each time.
     let mut start_ptr = self.as_ptr();
-    let original_length = self.length;
+    let old_length = self.length;
     // We update self.length inplace in the loop to track how many bytes
     // have been written. This means that when we perform the shift at the
     // end, self.length is already correct.
@@ -735,12 +762,14 @@ impl<const N: usize> Read for StaticVec<u8, N> {
         self.length -= read_length;
       }
     }
-    let total_read = original_length - self.length;
-    if self.length > 0 {
-      // TODO: find out if the optimizer elides the bounds check here. It
-      // should be able to, since the only non-const value is total_read,
-      // which is known to be <= self.length
-      self.as_mut_slice().copy_within(total_read.., 0);
+    let current_length = self.length;
+    let total_read = old_length - current_length;
+    if current_length > 0 {
+      unsafe {
+        self
+          .ptr_at_unchecked(total_read)
+          .copy_to(self.as_mut_ptr(), current_length)
+      };
     }
     Ok(total_read)
   }
